note
	description: "Node in a code tree."

deferred class
	NODE

feature -- Access

	probability: REAL_64
			-- Probability of any of the leafs in this subtree appearing in the text.

feature -- Traversal

	fill_codes (codes: V_TABLE [CHARACTER, STRING]; prefix_: STRING)
			-- Add paths to leafs of this subtree to `codes', if the path from root to this node is `prefix_'.
		require
			codes_exists: codes /= Void
			prefix_exists: prefix_ /= Void
		deferred
		end

	decode_from (encoding: STRING; start: INTEGER): TUPLE [char: CHARACTER; pos: INTEGER]
			-- Assuming an encoding of of one of the leafs in this subtree starts from `start' in `encoding',
			-- return the character at the leaf and the position where the encoding of the next character starts.
		require
			encoding_exists: encoding /= Void
			encoding_binary: across encoding as c all c.item = '0' or c.item = '1' end
			start_in_bounds: 1 <= start and start <= encoding.count
		deferred
		ensure
			pos_in_bounds: 1 <= Result.pos and Result.pos <= encoding.count + 1
		end

feature -- Output

	show (indent: STRING)
			-- Pretty-print this subtree.
		require
			indent_exists: indent /= Void
			indent_is_spaces: across indent as c all c.item.is_space end
		deferred
		end

invariant
	probability_in_range: 0.0 <= probability and probability <= 1.0

end
